module ButtonLatch(START, Run, Sending, CLK, RESET);
  input		START;
  output	Run;
  reg		Run;
  input		Sending;
  input		CLK;
  input		RESET;

/* Internals */
 reg [1:0] state;
 reg [1:0] next_state;

/* constants */
`define STARTUP 2'b00
`define WAIT	2'b10
`define GO	2'b11

/* clocked always */
 always @(posedge CLK) begin
 	if (RESET) begin
 		state = `STARTUP;
 	end
 	else begin /* !RESET */
 		state = next_state;
 	end
 end

 always @(START or Sending or state) begin
 	case(state)

 	`STARTUP: begin
 		if (START) next_state = `WAIT;
 		else next_state = `STARTUP;
		Run = 1'b0;
 	end

        /* WAIT state inserted to wait for Sending to come back from Pkt_Gen after one cycle */
	`WAIT: begin
 		if (Sending) next_state = `GO;
 		else next_state = `WAIT;
		Run = 1'b1;
 	end

 	`GO: begin
 		if (Sending) begin
			next_state = `GO;
			Run = 1'b1;
		end
 		else begin
			next_state = `STARTUP;
			Run = 1'b0;
		end
	end

 	endcase
 end

endmodule
