module Pkt_Gen(START, A15, A14, A13, A12, A11, A10, A9, A8, A7, A6, A5, A4, A3, A2, A1, A0, BAD_LEN, BAD_CHK, SerOut, CLK, RESET, Sending);
  input		START;
  input		A15;
  input		A14;
  input		A13;
  input		A12;
  input		A11;
  input		A10;
  input		A9;
  input		A8;
  input		A7;
  input		A6;
  input		A5;
  input		A4;
  input		A3;
  input		A2;
  input		A1;
  input		A0;
  input		BAD_LEN;
  input		BAD_CHK;
  output	SerOut;
  input		CLK;
  input		RESET;
  output	Sending;

/* Internals */
 reg [4:0] state;	/* regs */
 reg [4:0] next_state;	/* wires */
 
 reg [4:0] count;	/* reg */
 reg [4:0] next_count;	/* wires */

/* Checksum */
 reg [3:0] C;

/* 4 input nibbles */
 reg [3:0] nibble1;
 reg [3:0] nibble2;
 reg [3:0] nibble3;
 reg [3:0] nibble4;
 
/* outputs */
 reg out;		/* regs */
 reg next_out;		/* wires */

/* payload */
 reg [19:0] payload;	/* extra room for long packets */

/* 4 data nibbles  */ 
assign nibble1 = {A15, A14, A13, A12};
assign nibble2 = {A11, A10, A9, A8};
assign nibble3 = {A7, A6, A5, A4};
assign nibble4 = {A3, A2, A1, A0};

/* check sum computation with optional error */
assign C = (nibble1 + nibble2 + nibble3 + nibble4 + BAD_CHK);

/* assembled packet to be stuffed */
assign payload = {A15,A14,A13,A12,A11,A10,A9,A8,A7,A6,A5,A4,A3,A2,A1,A0,C[3],C[2],C[1],C[0]};

/* Constant Defn */
`define STARTUP 	5'b00000
`define HEADER0		5'b10000
`define HEADER1		5'b10001
`define HEADER2		5'b10010
`define HEADER3		5'b10011
`define PAYLOAD1	5'b11001
`define PAYLOAD2	5'b11010
`define STUFF1		5'b11101
`define STUFF2		5'b11110
`define PAYLOAD20	5'b11111
`define FOOTER0		5'b10100
`define FOOTER1		5'b10101
`define FOOTER2		5'b10110
`define FOOTER3		5'b10111
 
 
 /* state registers */
 always @( posedge CLK) begin
 	if( RESET ) begin
 		 state = `STARTUP;
		 out = 1'b0;
		 count = 0;
 	end
 	else begin
		 state = next_state;
		 out = next_out;
		 count = next_count;
	end		
  end
  
  /* combinational logic */
  always @(state or count or START) begin
  
  case(state)
  
  `STARTUP: begin	/* Sending = 0 */
  	if (START) begin 
  		next_state = `HEADER0;
  		next_out   = 1'b1;  /* first bit of start marker */
		next_count = 19 - BAD_LEN;  /* set up down_counter for payload */
					    /* with optional bad length (short by 1) */
    	end
  	
  	else begin
  		next_state = `STARTUP;
  		next_out   = 1'b0;   /* keep line zero if not sending */
  		next_count = 0;
  	end
  end
  	
  	
  `HEADER0: begin
  	next_state = `HEADER1;
  	next_out   = 1'b1;  /* second bit of start marker */
  	next_count = count;
  end
  
  	
  `HEADER1: begin
  	next_state = `HEADER2;
  	next_out   = 1'b1;  /* third bit of start marker */
  	next_count = count;
  end
  
  		
  `HEADER2: begin
  	next_state = `HEADER3;
  	next_out   = 1'b0;  /* fourth bit of start marker */
  	next_count = count;
  end
  
  		
  `HEADER3: begin
  	next_state = `PAYLOAD1;
  	next_out   = payload[count];
  	next_count = count - 1;
  end
  
  `PAYLOAD1: begin
	next_state = `PAYLOAD2;
	next_out   = payload[count];
	next_count = count - 1;
  end

  `PAYLOAD2: begin
	if (payload[count + 1] && payload [count + 2]) begin   /* stuff extra 0 */
		next_state = `STUFF1;
		next_out   = 1'b0;
                next_count = count;
	end
	else begin
	if (count == 0) next_state = `PAYLOAD20;
                   else next_state = `PAYLOAD2;
		next_out   = payload[count];
		next_count = count - 1;
	end
  end

  `STUFF1: begin
	if (count == 0) next_state = `PAYLOAD20;
                   else next_state = `STUFF2;
	next_out   = payload[count];
	next_count = count - 1;
  end

  `STUFF2: begin
	if (count == 0) next_state = `PAYLOAD20;
                   else next_state = `PAYLOAD2;
	next_out   = payload[count];
        next_count = count - 1;
  end
  
  `PAYLOAD20: begin
  	next_out = 1'b1;  /* first bit of end marker */
  	next_state = `FOOTER0;
  	next_count = count;	
  end
  		
  `FOOTER0: begin
  	next_out = 1'b1;  /* second bit of end marker */
  	next_state = `FOOTER1;
  	next_count = count;	
  end
  		
  `FOOTER1: begin
  	next_out = 1'b1;  /* third bit of end marker */
  	next_state = `FOOTER2;
  	next_count = count;	
  end
  		
  `FOOTER2: begin
  	next_out = 1'b0;  /* fourth bit of end marker */
  	next_state = `FOOTER3;
  	next_count = count;
  end

  `FOOTER3: begin
  	next_out = 1'b0;  /* put zero on line after end of packet */
  	next_state = `STARTUP;
  	next_count = 0;
  end

  endcase
 end
 
/* output assignments */

 assign Sending = (state != `STARTUP);
 assign SerOut  = out;

endmodule
